import requests
import six
import json
import operator
from suchwow import config
from decimal import Decimal


PICOWOW = Decimal('0.00000000001')

class Wallet(object):
    def __init__(self):
        self.host = config.WALLET_HOST
        self.port = config.WALLET_PORT
        self.proto = config.WALLET_PROTO
        self.username = config.WALLET_RPC_USER
        self.password = config.WALLET_RPC_PASS
        self.endpoint = '{}://{}:{}/json_rpc'.format(
            self.proto, self.host, self.port
        )
        self.auth = requests.auth.HTTPDigestAuth(
            self.username, self.password
        )

        try:
            r = self.height()
            height = r['height']
            self.connected = True
        except:
            self.connected = False

    def make_wallet_rpc(self, method, params={}):
        r = requests.get(
            self.endpoint,
            data=json.dumps({'method': method, 'params': params}),
            auth=self.auth,
            timeout=10
        )
        if 'error' in r.json():
            return r.json()['error']
        else:
            return r.json()['result']
    
    def store(self):
        self.make_wallet_rpc('store')

    def height(self):
        return self.make_wallet_rpc('get_height', {})

    def accounts(self):
        _accounts = self.make_wallet_rpc('get_accounts')
        return [i['account_index'] for i in _accounts['subaddress_accounts']]

    def new_account(self, label=None):
        _account = self.make_wallet_rpc('create_account', {'label': label})
        return _account['account_index']

    def addresses(self, account, addr_indices=None):
        qdata = {'account_index': account}
        if addr_indices:
            qdata['address_index'] = addr_indices
        _addresses = self.make_wallet_rpc('get_address', qdata)
        if 'message' in _addresses:
            return None
        addresses = [None] * (max(map(operator.itemgetter('address_index'), _addresses['addresses'])) + 1)
        for _addr in _addresses['addresses']:
            addresses[_addr['address_index']] = _addr['address']
        return addresses

    def get_address(self, account, address_indices=[]):
        qdata = {'account_index': account, 'address_index': address_indices}
        _addresses = self.make_wallet_rpc('get_address', qdata)
        return _addresses

    def new_address(self, account, label=None):
        data = {'account_index': account, 'label': label}
        _address = self.make_wallet_rpc('create_address', data)
        return (_address['address_index'], _address['address'])

    def transfers(self, account_index=0, address_indices=[], _in=True, _out=True):
        data = {
            'account_index': account_index,
            'subaddr_indices': address_indices,
            'in': _in,
            'out': _out
        }
        _transfers = self.make_wallet_rpc('get_transfers', data)
        return _transfers

    def balances(self, address_indices=[]):
        data = {
            'account_index': config.WALLET_ACCOUNT,
            'address_indices': address_indices
        }
        _balance = self.make_wallet_rpc('get_balance', data)
        return (_balance['balance'], _balance['unlocked_balance'])

    def transfer(self, dest_address, amount, priority, account):
        data = {
            'account_index': account,
            'destinations': [{'address': dest_address, 'amount': to_atomic(amount)}],
            'priority': priority,
            'unlock_time': 0,
            'get_tx_key': True,
            'get_tx_hex': True,
            'new_algorithm': True,
            'do_not_relay': False,
            'ring_size': 22
        }
        transfer = self.make_wallet_rpc('transfer', data)
        self.store()
        return transfer

    def incoming_transfers(self, transfer_type='all'):
        data = {
            'transfer_type': transfer_type,
            'account_index': config.WALLET_ACCOUNT            
        }
        transfers = self.make_wallet_rpc('incoming_transfers', data)
        return transfers


def to_atomic(amount):
    if not isinstance(amount, (Decimal, float) + six.integer_types):
        raise ValueError("Amount '{}' doesn't have numeric type. Only Decimal, int, long and "
                "float (not recommended) are accepted as amounts.")
    return int(amount * 10**11)

def from_atomic(amount):
    return (Decimal(amount) * PICOWOW).quantize(PICOWOW)

def as_wownero(amount):
    return float(Decimal(amount).quantize(PICOWOW))
